﻿/*
  Script:	main.js
  Gadget:	Website & URL Monitor
  Purpose:	Main gadget view
  Author:	Ben Coleman
  Update:	08 Oct 2007
  Notes:	Tested on Google Desktop v5.1
  Changes:	1.0.0.0	- Initial release
		1.1.0.0	- Added user & password to monitors and check for dodgy chars in monitor name
		1.1.1.0	- Monitors now sorted by name; and general code cleanup
		1.2.0.0	- Fixed some bugs, added last run time field, changed schedule from seconds to minutes
		1.3.0.0	- Resizes correctly after a restart
		1.5.0.0	- New font size options and update for Google Desktop v5.5
*/

// global vars
var monitors;					// array of monitors - created by de-serializing the mon_data
var details_view;				// details view pop-out
var disabled_checked = false;	// is the monitoring disabled (used by the menu)

//
// main gadget init function
//
function view_onOpen() 
{
	// hook call so our custom menu function is called 
	pluginHelper.onAddCustomMenuItems = addCustomMenuItems

	details_view = new DetailsView();
	monitors = new Array();

	options.putDefaultValue("show_time", true);
	options.putDefaultValue("alert_notify", true);
	options.putDefaultValue("alert_sound", true);
	options.putDefaultValue("disble_when_offline", false);
	options.putDefaultValue("mon_data", "");
	options.putDefaultValue("width", 200);
	options.putDefaultValue("height", 200);
	options.putDefaultValue("font_size", 10);
	options.putDefaultValue("font_bold", true);
	
	resizeTo(options.getValue("width"), options.getValue("height"));

	details_view.html_content = true;

	// FOR TESTING
	
	/*mon = new Monitor("Localhost", "http://localhost/", 20, 2, "wibble");
	monlist.appendElement(mon.toListItem());
	monitors.push(mon);
	mon = new Monitor("sis", "http://localhost:8888/SiteScope/cgi/go.exe/SiteScope?page=health&operation=List&account=administrator", 10, 3, null);
	monlist.appendElement(mon.toListItem());
	monitors.push(mon);	
	options.putValue("mon_data", serializeObjectArray(monitors));
	*/
	buildView();

	// set the monitors to run mode
	options.putValue("running", true);

	// invoke the main scheduling function every 1 minute
	setInterval("monitorRunner()", 60000);

	forceMonitorsRun();	
}

//
// called when user clicks OK in the options dialog
//
function view_onOptionChange()
{
	// weird check this, this callback function will get called for EVERY option value change 
	// - that would result in dozens of calls to buildView, but we want to invoke buildView once
	// - this traps the last value we change in the options_onOK() function so we only call buildView once
	if(event.propertyName == "show_time") {
		buildView();
		forceMonitorsRun();
	}
}

//
// render the monitors into the gadget list
//
function buildView()
{
	monitors = deserializeObjectArray(options.getValue("mon_data"), "Monitor");
	monlist.removeAllElements();
	for(var mi = 0; mi < monitors.length; mi++) {
		monlist.appendElement(monitors[mi].toListItem());
	}

	if(monitors.length == 0) {
		monlist.appendElement("<listitem enabled='false'><label bold='true' font='Calibri' size='9' color='#ffffff'>No monitors defined</label></listitem>");
		monlist.appendElement("<listitem enabled='false'><label enabled='true' cursor='hand' onclick='pluginHelper.ShowOptionsDialog()' bold='true' font='Calibri' size='9' color='#ffffff'>Click here to add a new monitor</label></listitem>");
	}
}

//
// add menu items to the main gadget menu
//
function addCustomMenuItems(menu)
{
	menu.AddItem("Run monitors now", 0, forceMonitorsRun);
	menu.AddItem("Disable monitoring",  disabled_checked ? gddMenuItemFlagChecked : 0, disableClicked); 
}

//
// add menu items to the main gadget menu
//
function disableClicked()
{
	disabled_checked = !disabled_checked;
	options.putValue("running", !disabled_checked);
}

//
// view has been resized - funky code to move the elements about
//
function view_onSize()
{
	w = view.width;
	h = view.height;
	
	tr.x = w - 32;
	bl.y = h - 32;
	br.x = w - 32;
	br.y = h - 32;
	t.width = w - 64;
	b.width = w - 64;
	b.y = h - 32;
	l.height = h - 64;
	r.height = h - 64;
	r.x = w - 32;

	mondiv.width = w - 40;
	mondiv.height = h - 40;
	
	options.putValue("width", w);
	options.putValue("height", h);
}

//
// check to make sure user can't make the gadget too small
//
function view_onSizing()
{
	if(event.height < 65) { event.returnValue = false }
	if(event.width < 65) { event.returnValue = false }
}

//
// main monitor scheduling "thread", called every 1 second
//
function monitorRunner()
{
	// check if the montiors are in a paused state
	if(!options.getValue("running")) return;

	for(m = 0; m < monitors.length; m++) {
		mon = monitors[m];
		mon.tick();
	}
}

//
// update the status of a monitor in the display, called from the Monitor object
//
function updateStatus(mon)
{
	var lab = mon.name;
	if(options.getValue("show_time")) {
		lab += " ("+mon.time+" ms)";
	}
	monlist.children.item(mon.name).children.item("img").src = "img/"+mon.status+".png";
	monlist.children.item(mon.name).children.item("img").tooltip = mon.status_msg;
	monlist.children.item(mon.name).children.item("lab").innerText = lab;
	monlist.children.item(mon.name).children.item("lab").tooltip = (mon.status_msg+"\nDouble click for more details");

	// check for alerts
	if(options.getValue("alert_notify")) {
		if((mon.status_prev != STATUS_NA) && (mon.status_prev < mon.status)) {
			var notify_item = new ContentItem();
			notify_item.heading = mon.name +" ~ "+ mon.statusText();
			notify_item.notifier_image = graphics.loadImage("img/notify/"+mon.status+".png");
			notify_item.snippet = mon.status_msg;
			pluginHelper.addContentItem(notify_item, gddItemDisplayAsNotification);
			
			if(options.getValue("alert_sound")) {
				var clip = framework.audio.open("alarms/alarm.wav", null);
				clip.play();
			}
		}
	}
}

//
// display the popup details view with info from the selected monitor
//
function monitorDetails()
{
	mon = monitors[monlist.selectedIndex];
	
	// Set the HTML to conatin the tooltip data (note we change the tabs into <BR>)
	details_html = "<html><body style='font-family:Calibri; font-size: 8pt'>" +
	"<table style='font-family:Calibri; font-size: 10pt'>" +
	"<tr><td style='font-weight:bold'>Monitor:</td><td>"+mon.name+"</td></tr>" +
	"<tr><td style='font-weight:bold'>Status:</td><td>"+mon.statusText()+"</td></tr>" +
	"<tr><td style='font-weight:bold'>URL:</td><td><a href='"+mon.url+"'>"+mon.url+"</a></td></tr>" +
	"<tr><td style='font-weight:bold'>Message:</td><td>"+mon.status_msg+"</td></tr>" +
	"<tr><td style='font-weight:bold'>Resp Time:</td><td>"+mon.time+" milli-secs</td></tr>" +
	"<tr><td style='font-weight:bold'>Last Run:</td><td>"+mon.last_run.getHours()+":"+mon.last_run.getMinutes()+":"+mon.last_run.getSeconds()+" "+mon.last_run.getDate()+"/"+(mon.last_run.getMonth()+1)+"/"+mon.last_run.getYear()+"</td></tr>" +
	"<tr><td style='font-weight:bold'>Check for:</td><td>"+(typeof(mon.check) != "undefined" ? mon.check : " &nbsp;")+"</td></tr>" +
	"<tr><td style='font-weight:bold'>Threshold:</td><td>"+mon.threshold+" milli-secs</td></tr>" +
	"<tr><td style='font-weight:bold'>Frequency:</td><td>Run every "+mon.freq+" minute</td></tr>" +
	"</table>" +
	"</body></html>";
	
	// hide the details view if it's shown
	try {
		pluginHelper.CloseDetailsView();
	} catch (e) {
	}
	// show the new details view
	details_view.setContent("", undefined, details_html, false, 0);
	pluginHelper.showDetailsView(details_view, mon.name, gddDetailsViewFlagNone, null);
}

//
// user has clicked force run
//
function forceMonitorsRun()
{
	for(var m = 0; m < monitors.length; m++) {
		monitors[m].runcounter = 0;
		monitors[m].tick();
	}	
}
