﻿/*
  Script:	options.js
  Gadget:	Website & URL Monitor
  Purpose:	Code to handle the gadget's options view
  Author:	Ben Coleman
  Update:	08 Oct 2007
  Notes:	Tested on Google Desktop v5.1
  Changes:	1.0.0.0	- Initial release
		1.1.0.0	- Added user & password to monitors and check for dodgy chars in monitor name
		1.1.1.0	- Monitors now sorted by name; and general code cleanup
		1.2.0.0	- Fixed some bugs, added last run time field, changed schedule from seconds to minutes
		1.5.0.0	- New font size options and update for Google Desktop v5.5
*/
var mon_count = 0;
var monitors;
var editing_index = -1;

//
// init the options view
//
function options_onOpen()
{
	// set this to pause the running of monitors
	options.putValue("running", false);

	// create the monitors array by de-serializing the mon_data string
	monitors = deserializeObjectArray(options.getValue("mon_data"), "Monitor");
	monitors.sort(monitorSort);

	// build the listbox
	for(var m = 0; m < monitors.length; m++) {
		list.appendElement("<item ondblclick='monitor_edit()'><label name='lab' font='Arial' size='10'>"+monitors[m].name+"</label></item>");
	}

	// read other options into the dialog GUI
	cb_notify.value = options.getValue("alert_notify");
	cb_alert.value = options.getValue("alert_sound");
	cb_show_time.value = options.getValue("show_time");
	cb_dis_offline.value = options.getValue("disble_when_offline");

	font_combo.selectedIndex = options.getValue("font_size") - 8;
	cb_font_bold.value = options.getValue("font_bold");

	notify_onChange();
}

//
// dialog is closing, set the monitor scheduler running again
//
function options_onClose()
{
	options.putValue("running", true);
}

//
// user has clicked OK to dismiss the options pane
//
function options_onOK()
{
	// sort the monitors by name and serialize back into the options object as a string
	monitors.sort(monitorSort);
	options.putValue("mon_data", serializeObjectArray(monitors));

	// store the rest of the options
	options.putValue("alert_notify", cb_notify.value);
	options.putValue("alert_sound", cb_alert.value);
	options.putValue("disble_when_offline", cb_dis_offline.value);
	options.putValue("font_bold", cb_font_bold.value);
	options.putValue("font_size", font_combo.selectedIndex + 8);
	options.putValue("show_time", cb_show_time.value);
}

//
// called when user has clicked the OK button in the monitor "sub-dialog"
//
function monitor_OK()
{
	// validate the field values; stop the user entering rubbish
	if(mon_name.value.trim().length == 0) { lab_err.innerText = "Monitor name can not be blank"; return; }
	if(mon_name.value.trim().indexOf("&") >= 0 || mon_name.value.trim().indexOf("'") >= 0 || mon_name.value.trim().indexOf("<") >= 0 || mon_name.value.trim().indexOf(">") >= 0) { lab_err.innerText = "Invalid character in monitor name"; return; }
	if(mon_url.value.trim().length == 0) { lab_err.innerText = "URL can not be blank"; return; }
	if(mon_thres.value.trim().length == 0) { lab_err.innerText = "Monitor threshold can not be blank"; return; }
	if(mon_freq.value.trim().length == 0) { lab_err.innerText = "Run frequency can not be blank"; return; }
	if(!isInteger(mon_thres.value.trim())) { lab_err.innerText = "Monitor threshold must be a positive integer"; return; }
	if(!isInteger(mon_freq.value.trim())) { lab_err.innerText = "Run frequency must be a positive integer"; return; }
	if(mon_thres.value.trim() < 0) { lab_err.innerText = "Monitor threshold can not be less than zero"; return; }
	if(mon_freq.value.trim() < 1) { lab_err.innerText = "Run frequency can not be less then 1 minute"; return; }

	// editing_index stores the monitor we are editing; -1 means we are adding a new monitor
	if(editing_index == -1) {
		// create a new monitor object, and push it into the array
		var m = new Monitor(mon_name.value.trim(), mon_url.value.trim(), mon_thres.value.trim(), mon_freq.value.trim(), mon_check.value, mon_user.value.trim(), mon_pass.value.trim());
		monitors.push(m);
		list.appendElement("<item ondblclick='monitor_edit()'><label name='lab' font='Arial' size='10'>"+m.name+"</label></item>");
	} else {
		// update the values of an existing monitor in the array
		var m = monitors[editing_index];
		m.name = mon_name.value.trim();
		m.url = mon_url.value.trim();
		m.threshold = mon_thres.value.trim();
		m.freq = mon_freq.value.trim();
		m.check = mon_check.value;
		m.user = mon_user.value;
		m.pass = mon_pass.value;
		list.selectedItem.children.item("lab").innerText = m.name;
	}

	// hide the monitor sub-dialog
	mondiv.visible = false;
	font_combo.visible = true;
}

//
// called when user has clicked the Cancel button in the monitor "sub-dialog"
//
function monitor_Cancel()
{
	// hide the monitor sub-dialog
	mondiv.visible = false;
	font_combo.visible = true;
}

//
// called when user has clicked the "add monitor" button, show the "sub-dialog"
//
function monitor_add()
{
	// clean up the sub-dialog for a new monitor
	lab_err.innerText = "";
	editing_index = -1;
	mon_name.value   = "New Monitor";
	mon_url.value    = "http://";
	mon_freq.value   = "15";
	mon_thres.value  = "2000";
	mon_check.value  = "";
	mon_user.value   = "";
	mon_pass.value   = "";

	// show the monitor sub-dialog
	mondiv.visible = true;
  font_combo.visible = false;
}

//
// called when user has clicked the "add monitor" button, show the "sub-dialog"
//
function monitor_edit()
{
	lab_err.innerText = "";

	// get the index of the monitor we are editing
	var sel = list.selectedIndex;
	editing_index = sel;

	// if it's in the array then populate the fields in the sub-dialog
	if(sel >= 0) {
		mon_name.value   = monitors[sel].name;
		mon_url.value    = monitors[sel].url;
		mon_freq.value   = monitors[sel].freq;
		mon_thres.value  = monitors[sel].threshold;
		mon_check.value  = monitors[sel].check;
		mon_user.value   = monitors[sel].user;
		mon_pass.value   = monitors[sel].pass;

		// ... then show the sub-dialog
		mondiv.visible = true;
		font_combo.visible = false;
	}
}

//
// called when user has clicked the "delete monitor" button
// 
function monitor_del()
{
	// TODO - in v5.5 add a user prompt here

	// get the selected monitor and remove from the array and list
	var sel = list.selectedIndex;
	if(sel >= 0) {
		monitors.splice(sel, 1);
		list.removeElement(list.selectedItem);
			
	}
}

//
// called when the user ticks/unticks the "notfiy me" checkbox
//
function notify_onChange()
{
	// grey out and disable GUI elements as required
	alert_on = cb_notify.value;
	if(alert_on) {
		//lab_sound.opacity = 255;
		cb_alert.opacity = 255; cb_alert.enabled = true;
	} else {
		//lab_sound.opacity = 80;
		cb_alert.opacity = 80; cb_alert.enabled = false;
	}
}